<?php

/*
	Phoronix Test Suite
	URLs: http://www.phoronix.com, http://www.phoronix-test-suite.com/
	Copyright (C) 2009 - 2010, Phoronix Media
	Copyright (C) 2009 - 2010, Michael Larabel
	pts_concise_display_mode.php: The batch / concise display mode

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

class pts_concise_display_mode implements pts_display_mode_interface
{
	private $tab = "    ";

	// Download / progress bits
	private $progress_tab_count = 1;
	private $progress_line_prefix = null;
	private $progress_char_count = 0;
	private $progress_char_pos = 0;
	private $progress_string_length = 0;
	private $progress_last_float = -1;

	// Test install bits
	private $test_install_pos = 0;
	private $test_install_count = 0;

	// Run bits
	private $expected_trial_run_count = 0;

	public function __construct()
	{

	}
	public function test_install_process($test_install_manager)
	{
		$this->test_install_pos = 0;
		$this->test_install_count = $test_install_manager->tests_to_install_count();

		echo "\n";
		echo $this->tab . $this->test_install_count . " Test" . ($this->test_install_count > 1 ? 's' : null) . " To Install\n";


		$download_size = 0;
		$download_total = 0;
		$cache_total = 0;
		$cache_size = 0;
		$install_size = 0;

		foreach($test_install_manager->get_test_run_requests() as $test_run_request)
		{
			$install_size += $test_run_request->test_profile->get_environment_size();

			foreach($test_run_request->get_download_objects() as $test_file_download)
			{
				switch($test_file_download->get_download_location_type())
				{
					case "IN_DESTINATION_DIR":
						// We don't really care about these files here since they are good to go
						break;
					case "LOCAL_DOWNLOAD_CACHE":
					case "REMOTE_DOWNLOAD_CACHE":
						$cache_size += $test_file_download->get_filesize();
						$cache_total++;
						break;
					default:
						$download_size += $test_file_download->get_filesize();
						$download_total++;
						break;
				}			
			}
		}


		if($download_total > 0)
		{
			echo $this->tab . $this->tab . $download_total . " File" . ($download_total > 1 ? 's' : null) . " To Download";

			if($download_size > 0)
			{
				echo " [" . pts_math::set_precision($download_size / 1048576, 2) . "MB]";
			}
			echo "\n";
		}

		if($cache_total > 0)
		{
			echo $this->tab . $this->tab . $cache_total . " File" . ($cache_total > 1 ? 's' : null) . " In Cache";

			if($cache_size > 0)
			{
				echo " [" . pts_math::set_precision($cache_size / 1048576, 2) . "MB]";
			}
			echo "\n";
		}

		if($install_size > 0)
		{
			echo $this->tab . $this->tab . ceil($install_size) . "MB Of Disk Space Is Needed\n";
		}

		echo "\n";
	}
	public function test_install_start($identifier)
	{
		$this->test_install_pos++;
		echo $this->tab . $identifier . ":\n";
		echo $this->tab . $this->tab . "Test Installation " . $this->test_install_pos . " of " . $this->test_install_count . "\n";
	}
	public function test_install_downloads($test_install_request)
	{
		$identifier = $test_install_request->test_profile->get_identifier();
		$download_packages = $test_install_request->get_download_objects();

		echo $this->tab . $this->tab . count($download_packages) . " File" . (isset($download_packages[1]) ? "s" : "") . " Needed";

		if(($size = $test_install_request->test_profile->get_download_size(false, 1048576)) > 0)
		{
			echo " [" . $size . " MB";

			if(($avg_speed = pts_download_speed_manager::get_average_download_speed()) > 0)
			{
				$avg_time = ($size * 1048576) / $avg_speed;
				echo " / " . pts_strings::format_time($avg_time, "SECONDS", true, 60);
			}

			echo ']';
		}

		echo "\n";
	}
	public function test_install_download_file($process, &$pts_test_file_download)
	{
		$expected_time = 0;
		$progress_prefix = null;

		switch($process)
		{
			case "DOWNLOAD_FROM_CACHE":
				$process_string = "Downloading From Cache";
				$progress_prefix = "Downloading";
				break;
			case "LINK_FROM_CACHE":
				$process_string = "Linking From Cache";
				break;
			case "COPY_FROM_CACHE":
				$process_string = "Copying From Cache";
				$progress_prefix = "Copying";
				break;
			case "FILE_FOUND":
				$process_string = "File Found";
				break;
			case "DOWNLOAD":
				$process_string = "Downloading";
				$progress_prefix = "Downloading";
				if(($avg_speed = pts_download_speed_manager::get_average_download_speed()) > 0 && ($this_size = $pts_test_file_download->get_filesize()) > 0)
				{
					$expected_time = $this_size / $avg_speed;
				}
				break;
		}

		$expected_time = is_numeric($expected_time) && $expected_time > 0 ? pts_strings::format_time($expected_time, "SECONDS", false, 60) : null;

		// TODO: handle if file-name is too long for terminal width
		$download_string = $this->tab . $this->tab . $process_string . ": " . $pts_test_file_download->get_filename();
		$download_size_string = $pts_test_file_download->get_filesize() > 0 ? " [" . pts_math::set_precision($pts_test_file_download->get_filesize() / 1048576, 2) . "MB]" : null;
		$offset_length = pts_client::terminal_width() > 1 ? pts_client::terminal_width() : pts_test_file_download::$longest_file_name_length;
		$offset_length = $offset_length - strlen($download_string) - strlen($download_size_string) - 2;

		if($offset_length < 2)
		{
			$offset_length = 2;
		}

		$download_string .= str_repeat(" ", ($offset_length - 2));
		$download_string .= $download_size_string;
		echo $download_string . "\n";

		$this->progress_line_prefix = $expected_time != null ? "Estimated Download Time: " . $expected_time : $progress_prefix;
		$this->progress_last_float = -1;
		$this->progress_tab_count = 2;
		$this->progress_string_length = strlen($download_string);
	}
	public function test_install_progress_start($process)
	{
		$this->progress_line_prefix = $process;
		$this->progress_last_float = -1;
		$this->progress_tab_count = 1;
		$this->progress_string_length = pts_client::terminal_width() > 1 ? pts_client::terminal_width() - 4 : 20;
		return;
	}
	public function test_install_progress_update($progress_float)
	{
		if($this->progress_last_float == -1)
		{
			$progress_prefix = str_repeat($this->tab, $this->progress_tab_count) . $this->progress_line_prefix . ' ';
			echo $progress_prefix;
			$this->progress_char_count = $this->progress_string_length - strlen($progress_prefix);
			$this->progress_char_pos = 0;
		}

		$char_current = floor($progress_float * $this->progress_char_count);

		if($char_current > $this->progress_char_pos && $char_current <= $this->progress_char_count)
		{
			echo str_repeat('.', $char_current - $this->progress_char_pos);
			$this->progress_char_pos = $char_current;
		}

		$this->progress_last_float = $progress_float;		
	}
	public function test_install_progress_completed()
	{
		echo $this->progress_last_float != -1 ? str_repeat('.', $this->progress_char_count - $this->progress_char_pos) . "\n" : null;
		$this->progress_last_float == -1;
	}
	public function test_install_begin($test_install_request)
	{
		if(($size = $test_install_request->test_profile->get_environment_size(false)) > 0)
		{
			echo $this->tab . $this->tab . "Installation Size: " . $size . " MB\n";
		}

		echo $this->tab . $this->tab . "Installing Test" . " @ " . date("H:i:s") . "\n";
		return;
	}
	public function test_install_output(&$to_output)
	{
		return;
	}
	public function test_install_error($error_string)
	{
		echo $this->tab . $this->tab . $this->tab . $error_string . "\n";
	}
	public function test_install_prompt($prompt_string)
	{
		echo $this->tab . $this->tab . $this->tab . $prompt_string;
	}
	public function test_run_process_start(&$test_run_manager)
	{
		return;
	}
	public function test_run_start(&$test_run_manager, &$test_result)
	{
		echo "\n\n" . $test_result->test_profile->get_title() . ":\n" . $this->tab . $test_result->test_profile->get_identifier();

		if(($test_description = $test_result->get_arguments_description()) != false)
		{
			echo " [" . pts_strings::swap_variables($test_description, array("pts_client", "environmental_variables")) . "]";
		}

		echo "\n";
		echo $this->tab . "Test " . $test_run_manager->get_test_run_position() . " of " . $test_run_manager->get_test_run_count_reported() . "\n";

		if(($remaining_length = $test_run_manager->get_estimated_run_time_remaining()) > 1)
		{
			echo $this->tab . "Estimated Time Remaining: " . pts_strings::format_time($remaining_length, "SECONDS", true, 60) . "\n";
		}

		$estimated_length = $test_result->test_profile->get_estimated_run_time();
		if($estimated_length > 1 && $estimated_length != $remaining_length)
		{
			echo $this->tab . "Estimated Test Run-Time: " . pts_strings::format_time($estimated_length, "SECONDS", true, 60) . "\n";
		}

		$this->expected_trial_run_count = $test_result->test_profile->get_times_to_run();
		echo $this->tab . "Expected Trial Run Count: " . $this->expected_trial_run_count;
	}
	public function test_run_message($message_string)
	{
		echo "\n" . $this->tab . $this->tab . $message_string . " @ " . date("H:i:s");
	}
	public function test_run_instance_header(&$test_result, $current_run, $total_run_count)
	{
		echo "\n" . $this->tab . $this->tab . "Started Run " . $current_run . " @ " . date("H:i:s");
	}
	public function test_run_instance_error($error_string)
	{
		echo "\n" . $this->tab . $this->tab . $error_string;
	}
	public function test_run_instance_output(&$to_output)
	{
		if((pts_c::$test_flags & pts_c::debug_mode))
		{
			echo $to_output;
		}

		return;
	}
	public function test_run_instance_complete(&$test_result)
	{
		if($test_result->test_profile->get_times_to_run() > $this->expected_trial_run_count)
		{
			// The run count must have been dynamically increased, so show the standard deviation
			echo " [Std. Dev: " . pts_math::set_precision(pts_math::percent_standard_deviation($test_result->test_result_buffer->get_values()), 2) . "%]";
		}
	}
	public function test_run_end(&$test_result)
	{
		echo "\n";

		if(in_array($test_result->test_profile->get_result_format(), array("NO_RESULT", "LINE_GRAPH", "IMAGE_COMPARISON")))
		{
			$end_print = null;
		}
		else if(in_array($test_result->test_profile->get_result_format(), array("PASS_FAIL", "MULTI_PASS_FAIL")))
		{
			$end_print = $this->tab . $this->tab . "Final: " . $test_result->get_result() . " (" . $test_result->test_profile->get_result_scale() . ")\n";
		}
		else
		{
			$end_print = "\n" . $this->tab . "Test Results:\n";

			foreach($test_result->test_result_buffer->get_values() as $result)
			{
				$end_print .= $this->tab . $this->tab . $result . "\n";
			}

			$end_print .= "\n" . $this->tab . pts_strings::result_format_to_string($test_result->test_profile->get_result_format()) . ": " . $test_result->get_result() . " " . $test_result->test_profile->get_result_scale();

			if($test_result->get_result() == 0)
			{
				$end_print .= "\n" . $this->tab . "This test failed to run properly.";
			}

			$end_print .= "\n";
		}

		echo $end_print;
	}
	public function test_run_error($error_string)
	{
		echo $this->tab . $this->tab . $error_string . "\n";
	}
	public function generic_prompt($prompt_string)
	{
		echo $this->tab . $prompt_string;
	}
	public function generic_heading($string, $ending_line_break = true)
	{
		static $shown_pts = false;

		if($shown_pts == false)
		{
			$string = pts_title() . "\n" . $string;
			$shown_pts = true;
		}

		if(!empty($string))
		{
			echo "\n";
			foreach(pts_strings::trim_explode("\n", $string) as $line_count => $line_string)
			{
				// ($line_count > 0 ? $this->tab : null) . 
				echo $line_string . "\n";
			}

			if($ending_line_break)
			{
				echo "\n";
			}
		}
	}
	public function generic_sub_heading($string)
	{
		if(!empty($string))
		{
			// To generate the "Phoronix Test Suite" heading string if not already done so
			pts_client::$display->generic_heading(null, false);

			foreach(pts_strings::trim_explode("\n", $string) as $line_string)
			{
				echo $this->tab . $line_string . "\n";
			}
		}
	}
	public function generic_error($string)
	{
		echo "\n";
		echo "ERROR: ";
		foreach(pts_strings::trim_explode("\n", $string) as $line_count => $line_string)
		{
			// ($line_count > 0 ? $this->tab : null) . 
			echo ($line_count > 0 ? "       " : null) . $line_string . "\n";
		}
		echo "\n";
	}
	public function generic_warning($string)
	{
		echo "\n";
		echo "WARNING: ";
		foreach(pts_strings::trim_explode("\n", $string) as $line_count => $line_string)
		{
			// ($line_count > 0 ? $this->tab : null) . 
			echo ($line_count > 0 ? "         " : null) . $line_string . "\n";
		}
		echo "\n";
	}
}

?>
